//	Copyright (c) Des Herriott 1993, 1994
//	              Erik Kunze   1995, 1996
//	              Jens Kilian  1998
//
//	Permission to use, copy, modify, distribute, and sell this software and its
//	documentation for any purpose is hereby granted without fee, provided that
//	the above copyright notice appear in all copies and that both that
//	copyright notice and this permission notice appear in supporting
//	documentation, and that the name of the copyright holder not be used in
//	advertising or publicity pertaining to distribution of the software without
//	specific, written prior permission.  The copyright holder makes no
//	representations about the suitability of this software for any purpose.  It
//	is provided "as is" without express or implied warranty.
//
//	THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
//	INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
//	EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//	CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
//	DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
//	TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
//	PERFORMANCE OF THIS SOFTWARE.

//	Command-line configuration options.

#ifndef _BZX_ZXCONFIG_H
#define _BZX_ZXCONFIG_H

#include <Point.h>

#include "screen.h"

struct ZXConfig {
	// Constructor, destructor.
	ZXConfig(int32 &argc, char **ppArgv, const char *pAppDir);
	~ZXConfig(void);
	
	// Standard options
	char *libDir;					// directory to check for support files

	// Display options
	BPoint location;				// initial window location
	bool mono;						// use only black and white

	char *kbdImage;				// image file for keyboard view
	BPoint kbdLocation;			// keyboard view location, size and status
	BPoint kbdSize;
	bool kbdShown;
	
	// Screen options
	int scale;						// Spectrum to BeOS screen scaling
	bool flashing;					// is flashing enabled?
	int refreshRate;				// screen refrsh rate

	// Emulation options
#ifdef DEBUG
	int debug;						// debug level
#endif
	int machine;					// what kind of Speccy to emulate
	int cycles;						// how many T-states between interrupts
	int ms;							// how many miliseconds between interrupts
#ifdef SLOWDOWN
	int slowdown;					// slowdown factor for very fast machines
#endif

	// Keyboard mapping
	uint8 actionKeys[NUM_KEYMAPPINGS];
	
	// Joystick options
	bool useJoy;					// try to use a joystick
	char *joyDevice;				// joystick device name (joystick[1-4])
	int joyTolerance;				// joystick displacement tolerance

	// Sound options
	bool useSound;					// use audio to emulate the speaker

	// Spectrum 48k options
	char *rom48;					// spectrum ROM image filename
	int issue;						// issue on 48K Spectrums
	
	// Spectrum 128k options
	char *rom128_0;				// 128K ROM 0
	char *rom128_1;				// 128K ROM 1

	// Spectrum +3 options
	char *rompl3_0;				// +3 ROM 0
	char *rompl3_1;				// +3 ROM 1
	char *rompl3_2;				// +3 ROM 2
	char *rompl3_3;				// +3 ROM 3
	char *disk_a;					// +3 floppy disk A:
	char *disk_b;					// +3 floppy disk B:

	// Interface 1 options
	bool if1_active;				// are we emulating if1 ?
	char *if1_rom;					// interface 1 ROM image filename
	bool translate_nl;			// Translate CR <-> LF on RS232 input?
	bool strip_nl;					// Strip CR on RS232 output?
	char *cartfiles[8];			// names of cartridge files	

	// Multiface 128 options
	bool mf128_active;			// switch multiface on/off
	char *mf128_rom;				// multiface 128 ROM

	static char *smpProgName;	// Name of program (for messages etc.)
	static ZXConfig *smpConf;	// Global configuration object.
	
private:
	ZXConfig(const ZXConfig &orig);
	ZXConfig &operator =(const ZXConfig &orig);
	
	// Process a single configuration option.
	bool ProcessOption(int option, const char *pArg);
	
	// Try to load resp. save configuration.
	void LoadSettings(void);
	void SaveSettings(void);
};

// Interfaces to the emulator code.
#define ProgName    (ZXConfig::smpProgName)
#define GETCFG(x)   (ZXConfig::smpConf->x)
#define SETCFG(x,y) (ZXConfig::smpConf->x = y)

#endif // _BZX_CONFIG_H
