/********************************************************************************
 * Copyright (c) Erik Kunze   1996
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the copyright holder not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 *
 * changed by EKU
 *******************************************************************************/

static char nas_c[] = "$Id: nas.c,v 3.0 1996/10/03 10:44:52 erik Exp $";

/********************************************************************************
 * nas.c - Audio emulation through the Network Audio System.
 * 
 * Currently we have:
 *******************************************************************************/

#include "z80.h"
#include "debug.h"
#include "util.h"
#include "nas.h"

/********************************************************************************
 * Defines
 *******************************************************************************/

/********************************************************************************
 * Forwards
 *******************************************************************************/

static void nasQuit(void);

/********************************************************************************
 * Globals
 *******************************************************************************/

/********************************************************************************
 * Locals
 *******************************************************************************/

static int nasOn = 0;
static int nasActive = 0;;
static int nasFd = -1;

/********************************************************************************
 * NasInit() - Initialize NAS system.
 *
 * Input:  audioOn - turn NAS on
 *
 * Return:  0 - o.k.
 *         -1 - error
 *******************************************************************************/

int
NasInit(int audioOn)
{
	OnQuit(nasQuit);

	/* NAS initailisation goes here */

	nasActive = 1;
	nasOn = 1;
		
	if (!audioOn)
	{
		NasOnOff();
	}
	
	return 0;
}

/********************************************************************************
 * nasQuit() - Shuddow the NAS system.
 *
 * Input:  -
 *
 * Return: -
 *******************************************************************************/

static void
nasQuit(void)
{
	if (nasFd != -1)
	{
		close(nasFd);
	}
	
	/* NAS closedown goes here */
}

/********************************************************************************
 * NasOnOff - Turns NAS sound on/off.
 *
 * Input:  -
 *
 * Return: -
 *******************************************************************************/

void
NasOnOff(void)
{
	if (nasActive)
	{
		nasOn = !nasOn;

		/* NAS start/stop goes here */
	}
}

/* EOF *************************************************************************/
