/********************************************************************************
 * Copyright (c) Erik Kunze  1995, 1996
 *               Jens Kilian 1998
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the copyright holder not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 *
 * changed by EKU
 * modified for BeZX JJK
 *******************************************************************************/

#ifndef _XZX_IO_H
#define _XZX_IO_H

/* $Id: io.h,v 3.0 1996/10/03 10:23:45 erik Exp $ */

#include "z80.h"

/********************************************************************************
 * Defines
 *******************************************************************************/

/* definitions for ULA */
#define PORT_ULA					0xfe	/* port address */
#define BIT_ULA					0x01	/* A1 = 0 -> select */
#define BIT_BORDER				0x07	/* border color bits */
#define BIT_MIC					0x08	/* tape out */
#define BIT_SPEAKER				0x10	/* speaker */
#define BIT_KEYBOARD				0x18	/* keyboard */
#define BIT_EAR					0x20	/* tape in */

/* definitions for ZX printer */
#define PORT_PRINTER				0xfb	/* port address */
#define BIT_PRINTER				0x02	/* A2 = 0 -> select */
#define BIT_SPEED					0x02	/* motor speed */
#define BIT_MOTOR					0x04	/* motor on/off */
#define BIT_STYLE					0x80	/* printer head in position */
#define BIT_ENCODER				0x01	/* printer ready */
#define BIT_PRESENT				0x40	/* printer present */
#define BIT_PAPER					0x80	/* out of paper */

/* definitions for kempston joystick */
#define PORT_KEMPSTON			0x1f	/* port address */
#define BIT_KEMPSTON				0x20	/* A5 = 0 -> select */
#define BIT_RIGHT 				0x01	/* joy right */
#define BIT_LEFT  				0x02	/* joy left */
#define BIT_DOWN  				0x04	/* joy down */
#define BIT_UP   				   0x08	/* joy up */
#define BIT_FIRE 				   0x10	/* joy fire */

/* definitions for all 128K and +3 ports */
#define PORT_128					0xfd	/* A1 = 0 -> select */

/* definitions for 128K memory management */
#define PORT_BANK128				0x7ffd	/* port address */
#define BIT_BANK128				0x8002	/* A15,1 = 0 -> select */
#define BIT_SELRAM				0x07	/* ram page on C000-FFFF */
#define BIT_SELSCREEN			0x08	/* screen select */
#define BIT_SELROM				0x10	/* rom select */
#define BIT_PAGING				0x20	/* enable/disable paging */

#define PAGING_DISABLED(port)	((port) & BIT_PAGING)

/* definitions for +3 memory management */
#define PORT_BANK3				0x1ffd	/* port address */
#define BIT_BANK3					0xe002	/* A15,14,13,1 = 0 -> select */
#define BIT_SPECIAL				0x01	/* paging mode */
#define BIT_SELRAM3				0x02	/* paging layout */
#define BIT_SELROM3				0x04	/* rom mode */
#define BIT_DISK					0x08	/* disk motor on/off */
#define BIT_STROBE				0x10	/* printer port strobe */

#define EXTENDED_RAM(port)		((port) & BIT_SPECIAL)

/* definitions for +3 additional ports */
#define PORT_CENTRONICS			0x0ffd	/* port address */
#define PORT_FDCCONTROL			0x2ffd	/* port address */
#define PORT_FDCDATA				0x3ffd	/* port address */
#define BIT_BUSY					0x01	/* printer busy */

/* definitions for AY-3-8912 sound generator */
#define PORT_SNDCONTROL			0xfffd	/* port address */
#define PORT_SNDDATA				0xbffd	/* port address */
#define BIT_SNDCHIP				0x8002	/* A15 = 1, A1 = 0 -> select */
#define BIT_SNDCONTROL			0x4000	/* A14 -> select control/data */
#define BIT_SNDDATA				BIT_SNDCONTROL

/* definitions for IF I */
#define PORT_MDCONTROL			0xef	/* port address */
#define PORT_MDDATA				0xe7	/* port address */
#define BIT_IF1					0x0010	/* A4 = 0 -> select */
#define BIT_MDCONTROL			0x0008	/* control/data */
#define BIT_MDDATA				BIT_MDCONTROL

/* definitions for Multiface 128 */
#define PORT_MF128IN				0xbf	/* port address */
#define PORT_MF128OUT			0x3f	/* port address */

/* defnitions for pseudo I/O */
#ifdef PSEUDO_IO
#define PSEUDO_IO_LOWBYTE		0xbf
#define PSEUDO_STDIN				(0x0000 | PSEUDO_IO_LOWBYTE)
#define PSEUDO_STDOUT			(0x0100 | PSEUDO_IO_LOWBYTE)
#define PSEUDO_STDERR			(0x0200 | PSEUDO_IO_LOWBYTE)
#endif

/********************************************************************************
 * Prototypes
 *******************************************************************************/

extern void OutBank678(uns8);
extern void OutByte(uns16, uns8);
extern uns8 InByte(uns16);

extern uns8 Last0x7FFD, Last0x1FFD, Last0xFFFD;
extern int ScreenSelect;
extern uns8 KeyPorts[9], InPorts[256];
extern int BasicRom;
extern int LastBorderColor;
extern unsigned char PSG[16];

#endif /* ! _XZX_IO_H */

/* EOF *************************************************************************/
