/********************************************************************************
 * Copyright (c) Des Herriott 1993, 1994
 *               Erik Kunze   1995, 1996
 *               Jens Kilian  1998
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the copyright holder not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Des Herriott
 *         Erik Kunze
 *
 * changed by EKU
 * modified for BeZX JJK
 *******************************************************************************/

static char rcsid[] = "$Id: joystick.c,v 3.0 1996/10/03 10:23:45 erik Exp $";

/********************************************************************************
 * joystick.c - Emulate a Kempston joystick.
 *******************************************************************************/

#include "ZXConfig.h"
#include "gamestick.h"
#include "debug.h"
#include "io.h"
#include "util.h"
#include "joystick.h"

/********************************************************************************
 * Defines
 *******************************************************************************/

/********************************************************************************
 * Forwards
 *******************************************************************************/

/********************************************************************************
 * Globals
 *******************************************************************************/

/********************************************************************************
 * Locals
 *******************************************************************************/

static BTSGameStick *pJoystick;
static bool joystickActive = false;

/********************************************************************************
 * quitJoy() - Perform a clean shutdown.
 *
 * Input:  -
 *
 * Return: -
 *******************************************************************************/

static void
quitJoy(void)
{
	joystickActive = false;
	delete pJoystick;
	pJoystick = 0;
}

/********************************************************************************
 * InitJoy() - Initialize and calibrate the joystick.
 *
 * Input:  -
 *
 * Return:  0 - o.k.
 *         -1 - error
 *******************************************************************************/

int
InitJoy(void)
{
	pJoystick = new BTSGameStick(GETCFG(joyDevice));
	if (!pJoystick->IsValid()) {
		delete pJoystick;
		XzxMesg(XZX_PWARN, "joystick initialisation failure!");
		return -1;
	}

#ifdef DEBUG
	if (GETCFG(debug) & D_JOY)
	{
		XzxMesg(XZX_DEBUG, "joystick initialisation successful.");
	}
#endif

	joystickActive = true;
	OnQuit(quitJoy);

	return 0;
}

/********************************************************************************
 * ReadJoy() - Read the current joystick position. If it's displaced by more
 *             than <joyTolerance> from centre, then OR in the appropriate
 *             direction bit.
 *
 * Input:  -
 *
 * Return: res - joystick direction and fire button status
 *******************************************************************************/

int 
ReadJoy(void)
{
	int res = 0;
	float x, y;
	const float tolerance = GETCFG(joyTolerance) / 100.0;
	
	if (!joystickActive)
	{
		return 0;
	}

	/* Get & decode the joystick return values */
	pJoystick->GetPosition(x, y);
	
	if (x < -tolerance) {
		res |= BIT_LEFT;
	} else if (x > tolerance) {
		res |= BIT_RIGHT;
	}

	if (y < -tolerance) {
		res |= BIT_UP;
	} else if (y > tolerance) {
		res |= BIT_DOWN;
	}

	if (pJoystick->IsButtonPressed(BTSGameStick::kButton1|BTSGameStick::kButton2)) {
		res |= BIT_FIRE;
	}

	return res;
}

/* EOF *************************************************************************/
