/********************************************************************************
 * Copyright (c) Des Herriott 1993, 1994
 *               Erik Kunze   1995, 1996
 *               Jens Kilian  1998
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the copyright holder not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Des Herriott
 *         Erik Kunze
 *
 * changed by EKU
 * modified for BeZX JJK
 *******************************************************************************/

static char rcsid[] = "$Id: main.c,v 3.1 1996/10/03 19:07:11 erik Exp $";

/********************************************************************************
 * main.c - Initial setup etc.
 *******************************************************************************/

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>

#include "ZXConfig.h"
#include "z80.h"
#include "mem.h"
#include "io.h"
#include "util.h"
#include "audio.h"
#include "dialog.h"
#include "fdc.h"
#include "emul.h"
#include "if1.h"
#include "joystick.h"
#include "screen.h"
#include "snapshot.h"
#include "main.h"

/********************************************************************************
 * Defines
 *******************************************************************************/

/********************************************************************************
 * Forwards
 *******************************************************************************/

static int initEmulator(int, char **);
static void quitEmulator(int);

/********************************************************************************
 * Globals
 *******************************************************************************/

/********************************************************************************
 * Locals
 *******************************************************************************/

/********************************************************************************
 * zx_main() - Main entry point of the emulator.
 *
 * Input:  argc - # of command line arguments
 *         argv - pointer to command line arguments
 *
 * Return: 0
 *******************************************************************************/

int
zx_main(int argc, char **argv)
{
	if (initEmulator(argc, argv) != -1)
	{
		Emulate();
	}

	Quit(1);

	/* NOTREACHED */

	return 0;
}

/********************************************************************************
 * initEmulator() - Initialize the emulator by calling the initialization
 *                  routine of each modul.
 *
 * Input:  ac - # of command line arguments
 *         av - pointer to command line arguments
 *
 * Return: -1 - error
 *          0 - o.k.
 *******************************************************************************/

static int
initEmulator(int ac, char **av)
{
	int i;

	/* Audio initialisation */
	AudioInit();

	if (DialogInit() == -1)
	{
		return -1;
	}

	/* Analogue joystick initialisation */
	if (GETCFG(useJoy) && InitJoy() == -1)
	{
		XzxMesg(XZX_WARN, "joystick initialisation failed");
		SETCFG(useJoy, false);
	}

	InitMemory();			/* Allocate the memory pages, no return on error */
	Z80_Init();				/* Initialize CPU							     */

	if (ReadROMs() == -1)	/* Read the ROM images						     */
	{
		return -1;			/* Couldn't read ROM images					     */
	}

	InitFDC();				/* Initialize the +3 floppy disk controller      */

	If1Init();				/* Initialize the Interface I				     */

	if (ac > 1)
	{
		/* Load snapshot file into spectrums memory */
		ReadSnapshot(av[1]);
	}
	else
	{
		/* To give the user a real spectrum feeling, we fill the display memory
		   with random data. It is a kind of powerup screen.*/
		srand(getpid());
		for (i = DFILE; i < (DFILE + PIXEL_LENGTH + ATTR_LENGTH); i++)
		{
			WR_BYTE(i, (uns8)rand());
		}
	}
	ScreenRefresh();

	signal(SIGINT,  quitEmulator);
	signal(SIGHUP,  quitEmulator);
	signal(SIGTERM, quitEmulator);
	signal(SIGQUIT, quitEmulator);

	return 0;
}

/********************************************************************************
 * quitEmulator() - Signal handler for interrupts.
 *
 * Input:  signo - signal #
 *
 * Return: -
 *******************************************************************************/

static void
quitEmulator(int /*signo*/)
{
	/* Shutdown the emulator */
	Quit(2);

	/* NOTREACHED */
}

/* EOF *************************************************************************/
