/********************************************************************************
 * Copyright (c) Des Herriott 1993, 1994
 *               Erik Kunze   1995, 1996
 *               Jens Kilian  1998
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the copyright holder not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Des Herriott
 *         Erik Kunze
 *
 * changed by EKU
 * modified for BeZX JJK
 *******************************************************************************/

static char edops_c[] = "$Id: z80_edops.c,v 3.0 1996/10/03 10:22:03 erik Exp $";

/********************************************************************************
 * edops.c - Operation prefixed by 0xED.
 *******************************************************************************/

/* in b,(c) */
case 0x40:
	Tstates += 12;
	IN(BC, B);
	break;

/* out (c),b */
case 0x41:
	Tstates += 12;
	OutByte(BC, B);
	break;

/* sbc hl,bc */
case 0x42:
	Tstates += 15;
	SBC_HL(BC);
	break;

/* ld (NN),bc */
case 0x43:
	Tstates += 20;
	WR_WORD(RD_WORD(PC), BC);
	PC += 2;
	break;

/* neg */
case 0x44:
case 0x4c:	/* inofficial */
case 0x54:	/* inofficial */
case 0x5c:	/* inofficial */
case 0x64:	/* inofficial */
case 0x6c:	/* inofficial */
case 0x74:	/* inofficial */
case 0x7c:	/* inofficial */
	Tstates += 8;
	work8 = A;
	A = 0;
	SUB(work8);
	break;

/* retn */
case 0x45:
case 0x55:	/* inofficial */
case 0x5d:	/* inofficial */
case 0x65:	/* inofficial */
case 0x6d:	/* inofficial */
case 0x75:	/* inofficial */
case 0x7d:	/* inofficial */
	Tstates += 14;
	POP(PC);
	IFF1 = IFF2;
	break;

/* im 0 */
case 0x46:
case 0x4e:	/* inofficial */
case 0x66:	/* inofficial */
case 0x6e:	/* inofficial */
	Tstates += 8;
	IM = 0;
	break;

/* ld i,a */
case 0x47:
	Tstates += 9;
	I = A;
	break;

/* in c,(c) */
case 0x48:
	Tstates += 12;
	IN(BC, C);
	break;

/* out (c),c */
case 0x49:
	Tstates += 12;
	OutByte(BC, C);
	break;

/* adc hl,bc */
case 0x4A:
	Tstates += 15;
	ADC_HL(BC);
	break;

/* ld bc,(NN) */
case 0x4B:
	Tstates += 20;
	BC = RD_WORD(RD_WORD(PC));
	PC += 2;
	break;

/* For our purposes, this is equivalent to ret */
/* reti */
case 0x4D:
	Tstates += 14;
	POP(PC);
	break;

/* ld r,a */
case 0x4F:
	Tstates += 9;
	R = A;
	break;

/* in d,(c) */
case 0x50:
	Tstates += 12;
	IN(BC, D);
	break;

/* out (c),d */
case 0x51:
	Tstates += 12;
	OutByte(BC, D);
	break;

/* sbc hl,de */
case 0x52:
	Tstates += 15;
	SBC_HL(DE);
	break;

/* ld (NN),de */
case 0x53:
	Tstates += 20;
	WR_WORD(RD_WORD(PC), DE);
	PC += 2;
	break;

/* im 1 */
case 0x56:
case 0x76:	/* inofficial */
	Tstates += 8;
	IM = 1;
	break;

/* ld a,i */
case 0x57:
	Tstates += 9;
	A = I;
	S_FLAG = ((A & 0x80) != 0);
	Z_FLAG = (A == 0);
	CLR(H_FLAG);
	P_FLAG = IFF2;
	CLR(N_FLAG);
	break;

/* in e,(c) */
case 0x58:
	Tstates += 12;
	IN(BC, E);
	break;

/* out (c),e */
case 0x59:
	Tstates += 12;
	OutByte(BC, E);
	break;

/* adc hl,de */
case 0x5A:
	Tstates += 15;
	ADC_HL(DE);
	break;

/* ld de,(NN) */
case 0x5B:
	Tstates += 20;
	DE = RD_WORD(RD_WORD(PC));
	PC += 2;
	break;

/* im 2 */
case 0x5E:
case 0x7e:	/* inofficial */
	Tstates += 8;
	IM = 2;
	break;

/* ld a,r */
case 0x5F:
	Tstates += 9;
	A = R;
	S_FLAG = ((A & 0x80) != 0);
	Z_FLAG = (A == 0);
	CLR(H_FLAG);
	P_FLAG = IFF2;
	CLR(N_FLAG);
	break;

/* in h,(c) */
case 0x60:
	Tstates += 12;
	IN(BC, H);
	break;

/* out (c),h */
case 0x61:
	Tstates += 12;
	OutByte(BC, H);
	break;

/* sbc hl,hl */
case 0x62:
	Tstates += 15;
	SBC_HL(HL);
	break;

/* ld (nn),hl */
case 0x63:	/* inofficial */
	Tstates += 20;
	WR_WORD(RD_WORD(PC), HL);
	PC += 2;
	break;

/* rrd */
case 0x67:
	Tstates += 18;
	work8 = RD_BYTE(HL);
	WR_BYTE(HL, (work8 >> 4) | (A << 4));
	A = (A & 0xf0) | (work8 & 0x0f);
	S_FLAG = ((A & 0x80) != 0);
	Z_FLAG = (A == 0);
	CLR(H_FLAG);
	P_FLAG = PARITY(A);
	CLR(N_FLAG);
	break;

/* in l,(c) */
case 0x68:
	Tstates += 12;
	IN(BC, L);
	break;

/* out (c),l */
case 0x69:
	Tstates += 12;
	OutByte(BC, L);
	break;

/* adc hl,hl */
case 0x6A:
	Tstates += 15;
	ADC_HL(HL);
	break;

/* ld hl,(NN) */
case 0x6B:	/* inofficial */
	Tstates += 20;
	HL = RD_WORD(RD_WORD(PC));
	PC += 2;
	break;

/* rld */
case 0x6F:
	Tstates += 18;
	work8 = RD_BYTE(HL);
	WR_BYTE(HL, (work8 << 4) | (A & 0x0f));
	A = (A & 0xf0) | (work8 >> 4);
	S_FLAG = ((A & 0x80) != 0);
	Z_FLAG = (A == 0);
	CLR(H_FLAG);
	P_FLAG = PARITY(A);
	CLR(N_FLAG);
	break;

/* in f,(c) */
case 0x70:
	Tstates += 12;
	IN(BC, work8);
	break;

/* out (c),0 */
case 0x71:	/* inofficial */
	Tstates += 12;
	OutByte(BC, 0);
	break;

/* sbc hl,sp */
case 0x72:
	Tstates += 15;
	SBC_HL(SP);
	break;

/* ld (NN),sp */
case 0x73:
	Tstates += 20;
	WR_WORD(RD_WORD(PC), SP);
	PC += 2;
	break;

/* nop */
case 0x77:	/* inofficial */
case 0x7f:	/* inofficial */
	Tstates += 8;
	break;

/* in a,(c) */
case 0x78:
	Tstates += 12;
	IN(BC, A);
	break;

/* out (c),a */
case 0x79:
	Tstates += 12;
	OutByte(BC, A);
	break;

/* adc hl,sp */
case 0x7A:
	Tstates += 15;
	ADC_HL(SP);
	break;

/* ld sp,(NN) */
case 0x7B:
	Tstates += 20;
	SP = RD_WORD(RD_WORD(PC));
	PC += 2;
	break;

/* ldi */
case 0xA0:
	Tstates += 16;
	WR_BYTE(DE, RD_BYTE(HL));
	DE++; HL++; BC--;
	V_FLAG = (BC != 0);
	CLR(H_FLAG);
	CLR(N_FLAG);
	break;

/* cpi */
case 0xA1:
	Tstates += 16;
	work8 = RD_BYTE(HL);
	CMP(work8);
	HL++; BC--;
	V_FLAG = (BC != 0);
	break;

/* ini */
case 0xA2:
	Tstates += 16;
	WR_BYTE(HL, InByte(BC));
	B--; HL++;
	Z_FLAG = (B == 0);
	SET(N_FLAG);
	break;

/* outi */
case 0xA3:
	Tstates += 16;
	B--;
	OutByte(BC, RD_BYTE(HL));
	HL++;
	Z_FLAG = (B == 0);
	SET(N_FLAG);
	break;

/* ldd */
case 0xA8:
	Tstates += 16;
	WR_BYTE(DE, RD_BYTE(HL));
	DE--; HL--; BC--;
	V_FLAG = (BC != 0);
	CLR(H_FLAG);
	CLR(N_FLAG);
	break;

/* cpd */
case 0xA9:
	Tstates += 16;
	work8 = RD_BYTE(HL);
	CMP(work8);
	HL--; BC--;
	V_FLAG = (BC != 0);
	break;

/* ind */
case 0xAA:
	Tstates += 16;
	WR_BYTE(HL, InByte(BC));
	B--; HL--;
	Z_FLAG = (B == 0);
	SET(N_FLAG);
	break;

/* outd */
case 0xAB:
	Tstates += 16;
	B--;
	OutByte(BC, RD_BYTE(HL));
	HL--;
	Z_FLAG = (B == 0);
	SET(N_FLAG);
	break;

/* ldir */
case 0xB0:
	Tstates += 16;
	WR_BYTE(DE, RD_BYTE(HL));
	HL++; DE++; BC--;
	V_FLAG = (BC != 0);
	CLR(H_FLAG);
	CLR(N_FLAG);
	if (TST(V_FLAG))
	{
		Tstates += 5;
		PC -= 2;
	}
	break;

/* cpir */
case 0xB1:
	Tstates += 16;
	work8 = RD_BYTE(HL);
	CMP(work8);
	HL++; BC--;
	V_FLAG = (BC != 0);
	if (TST(V_FLAG) && !TST(Z_FLAG))
	{
		Tstates += 5;
		PC -= 2;
	}
	break;

/* inir */
case 0xB2:
	Tstates += 16;
	WR_BYTE(HL, InByte(BC));
	B--; HL++;
	Z_FLAG = (B == 0);
	SET(N_FLAG);
	if (!TST(Z_FLAG))
	{
		Tstates += 5;
		PC -= 2;
	}
	break;

/* otir */
case 0xB3:
	Tstates += 16;
	B--;
	OutByte(BC, RD_BYTE(HL));
	HL++;
	Z_FLAG = (B == 0);
	SET(N_FLAG);
	if (!TST(Z_FLAG))
	{
		Tstates += 5;
		PC -= 2;
	}
	break;

/* lddr */
case 0xB8:
	Tstates += 16;
	WR_BYTE(DE, RD_BYTE(HL));
	HL--; DE--; BC--;
	V_FLAG = (BC != 0);
	CLR(H_FLAG);
	CLR(N_FLAG);
	if (TST(V_FLAG))
	{
		Tstates += 5;
		PC -= 2;
	}
	break;

/* cpdr */
case 0xB9:
	Tstates += 16;
	work8 = RD_BYTE(HL);
	CMP(work8);
	HL--; BC--;
	V_FLAG = (BC != 0);
	if (TST(V_FLAG) && !TST(Z_FLAG))
	{
		Tstates += 5;
		PC -= 2;
	}
	break;

/* indr */
case 0xBA:
	Tstates += 16;
	WR_BYTE(HL, InByte(BC));
	B--; HL--;
	Z_FLAG = (B == 0);
	SET(N_FLAG);
	if (!TST(Z_FLAG))
	{
		Tstates += 5;
		PC -= 2;
	}
	break;

/* otdr */
case 0xBB:
	Tstates += 16;
	B--;
	OutByte(BC, RD_BYTE(HL));
	HL--;
	Z_FLAG = (B == 0);
	SET(N_FLAG);
	if (!TST(Z_FLAG))
	{
		Tstates += 5;
		PC -= 2;
	}
	break;

/* Now some pseudo-ops with special meaning to xzx */

#ifdef DEBUG

/* Debug trap */
case 0x00:
	Tstates += 8;
	XzxMesg(XZX_DEBUG, "ED 00 - processor trap");
	DumpCpuState();
	fprintf(stderr, "  <q> to quit, any other key to continue: ");
	do
	{
		work8 = getchar();
		if (work8 == 'q' || work8 == 'Q')
		{
			Quit(0);
		}
	} while (work8 == 0xff);
	break;

#endif

#ifdef XBELL_AUDIO

/* Trap the ROM and ring the server bell */
case 0xFA:
	Tstates += 8;
	if (GETCFG(bellVol))
	{
		int f, t;

		f = 1750000 / ((HL + 30) << 2);
		t = (1000 * DE) / f;
		RingBell(f, (int)t);
	}
	break;

#endif

/* Level Loader trap */
case 0xFB:
	Tstates += 8;
	LevelLoaderTrap();
	break;

case 0xFC:
	Tstates += 8;
	Loader();
	break;

case 0xFD:
	Tstates += 8;
	Saver();
	break;

/* Interface I RS232 I/O traps */
case 0xFE:
	Tstates += 8;
	RS232IpTrap();
	break;

case 0xFF:
	Tstates += 8;
	RS232OpTrap();
	break;

/* The ED opcodes in the range 00-3F and 80-FF (except for the block
   instructions of course) do nothing at all but taking up 8 T states and
   incrementing the R register by 2. */

/* unimplemented */
default:
	Tstates += 8;

#ifdef DEBUG
	if (GETCFG(debug) & D_ILLOPS)
	{
		XzxMesg(XZX_DEBUG, "unimplemented instruction %02x %02x at %04x",
				RD_BYTE(PC - 2), RD_BYTE(PC - 1), PC -2);
	}
#endif

	break;

/* EOF *************************************************************************/
