/********************************************************************************
 * Copyright (c) Des Herriott 1993, 1994
 *               Erik Kunze   1995, 1996
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the copyright holder not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Des Herriott
 *         Erik Kunze
 *
 * changed by EKU
 *******************************************************************************/

static char ops_c[] = "$Id: z80_ops.c,v 3.0 1996/10/03 10:22:03 erik Exp $";

/********************************************************************************
 * z80ops.c - Contains the regular Z80 operations.
 *******************************************************************************/

/* nop */
case 0x00:
	Tstates += 4;
	break;

/* ld bc,NN */
case 0x01:
	Tstates += 10;
	BC = RD_WORD(PC);
	PC += 2;
	break;

/* ld (bc),a */
case 0x02:
	Tstates += 7;
	WR_BYTE(BC, A);
	break;

/* inc bc */
case 0x03:
	Tstates += 6;
	BC++;
	break;

/* inc b */
case 0x04:
	Tstates += 4;
	INC(B);
	break;

/* dec b */
case 0x05:
	Tstates += 4;
	DEC(B);
	break;

/* ld b,N */
case 0x06:
	Tstates += 7;
	B = RD_BYTE(PC++);
	break;

/* rlca */
case 0x07:
	Tstates += 4;
	C_FLAG = ((A & 0x80) != 0);
	A = (A << 1) | C_FLAG;
	CLR(H_FLAG);
	CLR(N_FLAG);
	break;

/* ex af,af' */
case 0x08:
	Tstates += 4;
	Z80_StoreFlags();
	work16 = AF; AF = AF1; AF1 = work16;
	Z80_RetrieveFlags();
	break;

/* add hl,bc */
case 0x09:
	Tstates += 11;
	ADD_HL(BC);
	break;

/* ld a,(bc) */
case 0x0A:
	Tstates += 7;
	A = RD_BYTE(BC);
	break;

/* dec bc */
case 0x0B:
	Tstates += 6;
	BC--;
	break;

/* inc c */
case 0x0C:
	Tstates += 4;
	INC(C);
	break;

/* dec c */
case 0x0D:
	Tstates += 4;
	DEC(C);
	break;

/* ld c,N */
case 0x0E:
	Tstates += 7;
	C = RD_BYTE(PC++);
	break;

/* rrca */
case 0x0F:
	Tstates += 4;
	C_FLAG = A & 0x01;
	A = (A >> 1) | (C_FLAG << 7);
	CLR(H_FLAG);
	CLR(N_FLAG);
	break;

/* djnz D */
case 0x10:
	if (--B)
	{
		Tstates += 13;
		PC += (sgn8)RD_BYTE(PC) + 1;
	}
	else 
	{
		Tstates += 8;
		PC++;
	}
	break;

/* ld de,NN */
case 0x11:
	Tstates += 10;
	DE = RD_WORD(PC);
	PC += 2;
	break;

/* ld (de),a */
case 0x12:
	Tstates += 7;
	WR_BYTE(DE, A);
	break;

/* inc de */
case 0x13:
	Tstates += 6;
	DE++;
	break;

/* inc d */
case 0x14:
	Tstates += 4;
	INC(D);
	break;

/* dec d */
case 0x15:
	Tstates += 4;
	DEC(D);
	break;

/* ld d,N */
case 0x16:
	Tstates += 7;
	D = RD_BYTE(PC++);
	break;

/* rla */
case 0x17:
	Tstates += 4;
	work8 = C_FLAG;
	C_FLAG = ((A & 0x80) != 0);
	A = (A << 1) | work8;
	CLR(H_FLAG);
	CLR(N_FLAG);
	break;

/* jr D */
case 0x18:
	Tstates += 12;
	PC += (sgn8)RD_BYTE(PC) + 1;
	break;

/* add hl,de */
case 0x19:
	Tstates += 11;
	ADD_HL(DE);
	break;

/* ld a,(de) */
case 0x1A:
	Tstates += 7;
	A = RD_BYTE(DE);
	break;

/* dec de */
case 0x1B:
	Tstates += 6;
	DE--;
	break;

/* inc e */
case 0x1C:
	Tstates += 4;
	INC(E);
	break;

/* dec e */
case 0x1D:
	Tstates += 4;
	DEC(E);
	break;

/* ld e,N */
case 0x1E:
	Tstates += 7;
	E = RD_BYTE(PC++);
	break;

/* rra */
case 0x1F:
	Tstates += 4;
	work8 = C_FLAG;
	C_FLAG = A & 0x01;
	A = (A >> 1) | (work8 << 7);
	CLR(H_FLAG);
	CLR(N_FLAG);
	break;

/* jr nz,D */
case 0x20:
	if (!TST(Z_FLAG))
	{
		Tstates += 12;
		PC += (sgn8)RD_BYTE(PC) + 1;
	}
	else
	{
		Tstates += 7;
		PC++;
	}
	break;

/* ld hl,NN */
case 0x21:
	Tstates += 10;
	HL = RD_WORD(PC);
	PC += 2;
	break;

/* ld (NN),hl */
case 0x22:
	Tstates += 16;
	WR_WORD(RD_WORD(PC), HL);
	PC += 2;
	break;

/* inc hl */
case 0x23:
	Tstates += 6;
	HL++;
	break;

/* inc h */
case 0x24:
	Tstates += 4;
	INC(H);
	break;

/* dec h */
case 0x25:
	Tstates += 4;
	DEC(H);
	break;

/* ld h,N */
case 0x26:
	Tstates += 7;
	H = RD_BYTE(PC++);
	break;

/* daa */
case 0x27:
	Tstates += 4;
	sub_ = N_FLAG;
	high = (A & 0xf0) >> 4;
	low  = A & 0x0f;

	if (!TST(N_FLAG))
	{
		if (!TST(C_FLAG))
		{
			if (!TST(H_FLAG))
			{
				if (low < 10)
				{
					if (high < 10)
					{
						add = 0x00;
						carry = 0;
					}
					else
					{
						add = 0x60;
						carry = 1;
					}
				}
				else
				{
					if (high < 9)
					{
						add = 0x06;
						carry = 0;
					}
					else
					{
						add = 0x66;
						carry = 1;
					}
				}
			}
			else
			{	/* H_FLAG */
				if (high < 10)
				{
					add = 0x06;
					carry = 0;
				}
				else
				{
					add = 0x66;
					carry = 1;
				}
			}	/* H_FLAG */
		}
		else
		{	/* C_FLAG */
			carry = 1;
			add = (!TST(H_FLAG) && low < 10) ? 0x60 : 0x66;
		}	/* C_FLAG */
	}
	else
	{	/* N_FLAG */
		if (!(carry = TST(C_FLAG)))
		{
			add = TST(H_FLAG) ? 0xfa : 0x00;
		}
		else
		{
			add = TST(H_FLAG) ? 0x9a : 0xa0;
		}
	} /* N_FLAG */

	ADD(add);
	N_FLAG = sub_;
	P_FLAG = PARITY(A);
	C_FLAG = carry;
	break;

/* jr z,D */
case 0x28:
	if (TST(Z_FLAG))
	{
		Tstates += 12;
		PC += (sgn8)RD_BYTE(PC) + 1;
	}
	else
	{
		Tstates += 7;
		PC++;
	}
	break;

/* add hl,hl */
case 0x29:
	Tstates += 11;
	ADD_HL(HL);
	break;

/* ld hl,(NN) */
case 0x2A:
	Tstates += 16;
	HL = RD_WORD(RD_WORD(PC));
	PC += 2;
	break;

/* dec hl */
case 0x2B:
	Tstates += 6;
	HL--;
	break;

/* inc l */
case 0x2C:
	Tstates += 4;
	INC(L);
	break;

/* dec l */
case 0x2D:
	Tstates += 4;
	DEC(L);
	break;

/* ld l,N */
case 0x2E:
	Tstates += 7;
	L = RD_BYTE(PC++);
	break;

/* cpl */
case 0x2F:
	Tstates += 4;
	A ^= 0xff;
	SET(H_FLAG);
	SET(N_FLAG);
	break;

/* jr nc,D */
case 0x30:
	if(!TST(C_FLAG)) {
		Tstates += 12;
		PC += (sgn8)RD_BYTE(PC) + 1;
	}
	else {
		Tstates += 7;
		PC++;
	}
	break;

/* ld sp,NN */
case 0x31:
	Tstates += 10;
	SP = RD_WORD(PC);
	PC += 2;
	break;

/* ld (NN),a */
case 0x32:
	Tstates += 13;
	WR_BYTE(RD_WORD(PC), A);
	PC += 2;
	break;

/* inc sp */
case 0x33:
	Tstates += 6;
	SP++;
	break;

/* inc (hl) */
case 0x34:
	Tstates += 11;
	work8 = RD_BYTE(HL);
	INC(work8);
	WR_BYTE(HL, work8);
	break;

/* dec (hl) */
case 0x35:
	Tstates += 11;
	work8 = RD_BYTE(HL);
	DEC(work8);
	WR_BYTE(HL, work8);
	break;

/* ld (hl),N */
case 0x36:
	Tstates += 10;
	WR_BYTE(HL, RD_BYTE(PC++));
	break;

/* scf */
case 0x37:
	Tstates += 4;
	CLR(H_FLAG);
	CLR(N_FLAG);
	SET(C_FLAG);
	break;

/* jr c,D */
case 0x38:
	if (TST(C_FLAG))
	{
		Tstates += 12;
		PC += (sgn8)RD_BYTE(PC) + 1;
	}
	else {
		Tstates += 7;
		PC++;
	}
	break;

/* add hl,sp */
case 0x39:
	Tstates += 11;
	ADD_HL(SP);
	break;

/* ld a,(NN) */
case 0x3A:
	Tstates += 13;
	A = RD_BYTE(RD_WORD(PC));
	PC += 2;
	break;

/* dec sp */
case 0x3B:
	Tstates += 6;
	SP--;
	break;

/* inc a */
case 0x3C:
	Tstates += 4;
	INC(A);
	break;

/* dec a */
case 0x3D:
	Tstates += 4;
	DEC(A);
	break;

/* ld a,N */
case 0x3E:
	Tstates += 7;
	A = RD_BYTE(PC++);
	break;

/* ccf */
case 0x3F:
	Tstates += 4;
	H_FLAG = C_FLAG;
	CLR(N_FLAG);
	CPL(C_FLAG);
	break;

/* ld b,b */
case 0x40:
	Tstates += 4;
	break;

/* ld b,c */
case 0x41:
	Tstates += 4;
	B = C;
	break;

/* ld b,d */
case 0x42:
	Tstates += 4;
	B = D;
	break;

/* ld b,e */
case 0x43:
	Tstates += 4;
	B = E;
	break;

/* ld b,h */
case 0x44:
	Tstates += 4;
	B = H;
	break;

/* ld b,l */
case 0x45:
	Tstates += 4;
	B = L;
	break;

/* ld b,(hl) */
case 0x46:
	Tstates += 7;
	B = RD_BYTE(HL);
	break;

/* ld b,a */
case 0x47:
	Tstates += 4;
	B = A;
	break;

/* ld c,b */
case 0x48:
	Tstates += 4;
	C = B;
	break;

/* ld c,c */
case 0x49:
	Tstates += 4;
	break;

/* ld c,d */
case 0x4A:
	Tstates += 4;
	C = D;
	break;

/* ld c,e */
case 0x4B:
	Tstates += 4;
	C = E;
	break;

/* ld c,h */
case 0x4C:
	Tstates += 4;
	C = H;
	break;

/* ld c,l */
case 0x4D:
	Tstates += 4;
	C = L;
	break;

/* ld c,(hl) */
case 0x4E:
	Tstates += 7;
	C = RD_BYTE(HL);
	break;

/* ld c,a */
case 0x4F:
	Tstates += 4;
	C = A;
	break;

/* ld d,b */
case 0x50:
	Tstates += 4;
	D = B;
	break;

/* ld d,c */
case 0x51:
	Tstates += 4;
	D = C;
	break;

/* ld d,d */
case 0x52:
	Tstates += 4;
	break;

/* ld d,e */
case 0x53:
	Tstates += 4;
	D = E;
	break;

/* ld d,h */
case 0x54:
	Tstates += 4;
	D = H;
	break;

/* ld d,l */
case 0x55:
	Tstates += 4;
	D = L;
	break;

/* ld d,(hl) */
case 0x56:
	Tstates += 7;
	D = RD_BYTE(HL);
	break;

/* ld d,a */
case 0x57:
	Tstates += 4;
	D = A;
	break;

/* ld e,b */
case 0x58:
	Tstates += 4;
	E = B;
	break;

/* ld e,c */
case 0x59:
	Tstates += 4;
	E = C;
	break;

/* ld e,d */
case 0x5A:
	Tstates += 4;
	E = D;
	break;

/* ld e,e */
case 0x5B:
	Tstates += 4;
	break;

/* ld e,h */
case 0x5C:
	Tstates += 4;
	E = H;
	break;

/* ld e,l */
case 0x5D:
	Tstates += 4;
	E = L;
	break;

/* ld e,(hl) */
case 0x5E:
	Tstates += 7;
	E = RD_BYTE(HL);
	break;

/* ld e,a */
case 0x5F:
	Tstates += 4;
	E = A;
	break;

/* ld h,b */
case 0x60:
	Tstates += 4;
	H = B;
	break;

/* ld h,c */
case 0x61:
	Tstates += 4;
	H = C;
	break;

/* ld h,d */
case 0x62:
	Tstates += 4;
	H = D;
	break;

/* ld h,e */
case 0x63:
	Tstates += 4;
	H = E;
	break;

/* ld h,h */
case 0x64:
	Tstates += 4;
	break;

/* ld h,l */
case 0x65:
	Tstates += 4;
	H = L;
	break;

/* ld h,(hl) */
case 0x66:
	Tstates += 7;
	H = RD_BYTE(HL);
	break;

/* ld h,a */
case 0x67:
	Tstates += 4;
	H = A;
	break;

/* ld l,b */
case 0x68:
	Tstates += 4;
	L = B;
	break;

/* ld l,c */
case 0x69:
	Tstates += 4;
	L = C;
	break;

/* ld l,d */
case 0x6A:
	Tstates += 4;
	L = D;
	break;

/* ld l,e */
case 0x6B:
	Tstates += 4;
	L = E;
	break;

/* ld l,h */
case 0x6C:
	Tstates += 4;
	L = H;
	break;

/* ld l,l */
case 0x6D:
	Tstates += 4;
	break;

/* ld l,(hl) */
case 0x6E:
	Tstates += 7;
	L = RD_BYTE(HL);
	break;

/* ld l,a */
case 0x6F:
	Tstates += 4;
	L = A;
	break;

/* ld (hl),b */
case 0x70:
	Tstates += 7;
	WR_BYTE(HL, B);
	break;

/* ld (hl),c */
case 0x71:
	Tstates += 7;
	WR_BYTE(HL, C);
	break;

/* ld (hl),d */
case 0x72:
	Tstates += 7;
	WR_BYTE(HL, D);
	break;

/* ld (hl),e */
case 0x73:
	Tstates += 7;
	WR_BYTE(HL, E);
	break;

/* ld (hl),h */
case 0x74:
	Tstates += 7;
	WR_BYTE(HL, H);
	break;

/* ld (hl),l */
case 0x75:
	Tstates += 7;
	WR_BYTE(HL, L);
	break;

/* halt */
case 0x76:
	Tstates += 4;
	PC--;
	break;

/* ld (hl),a */
case 0x77:
	Tstates += 7;
	WR_BYTE(HL, A);
	break;

/* ld a,b */
case 0x78:
	Tstates += 4;
	A = B;
	break;

/* ld a,c */
case 0x79:
	Tstates += 4;
	A = C;
	break;

/* ld a,d */
case 0x7A:
	Tstates += 4;
	A = D;
	break;

/* ld a,e */
case 0x7B:
	Tstates += 4;
	A = E;
	break;

/* ld a,h */
case 0x7C:
	Tstates += 4;
	A = H;
	break;

/* ld a,l */
case 0x7D:
	Tstates += 4;
	A = L;
	break;

/* ld a,(hl) */
case 0x7E:
	Tstates += 7;
	A = RD_BYTE(HL);
	break;

/* ld a,a */
case 0x7F:
	Tstates += 4;
	break;

/* add a,b */
case 0x80:
	Tstates += 4;
	ADD(B);
	break;

/* add a,c */
case 0x81:
	Tstates += 4;
	ADD(C);
	break;

/* add a,d */
case 0x82:
	Tstates += 4;
	ADD(D);
	break;

/* add a,e */
case 0x83:
	Tstates += 4;
	ADD(E);
	break;

/* add a,h */
case 0x84:
	Tstates += 4;
	ADD(H);
	break;

/* add a,l */
case 0x85:
	Tstates += 4;
	ADD(L);
	break;

/* add a,(hl) */
case 0x86:
	Tstates += 7;
	work8 = RD_BYTE(HL);
	ADD(work8);
	break;

/* add a,a */
case 0x87:
	Tstates += 4;
	ADD(A);
	break;

/* adc a,b */
case 0x88:
	Tstates += 4;
	ADC(B);
	break;

/* adc a,c */
case 0x89:
	Tstates += 4;
	ADC(C);
	break;

/* adc a,d */
case 0x8A:
	Tstates += 4;
	ADC(D);
	break;

/* adc a,e */
case 0x8B:
	Tstates += 4;
	ADC(E);
	break;

/* adc a,h */
case 0x8C:
	Tstates += 4;
	ADC(H);
	break;

/* adc a,l */
case 0x8D:
	Tstates += 4;
	ADC(L);
	break;

/* adc a,(hl) */
case 0x8E:
	Tstates += 7;
	work8 = RD_BYTE(HL);
	ADC(work8);
	break;

/* adc a,a */
case 0x8F:
	Tstates += 4;
	ADC(A);
	break;

/* sub b */
case 0x90:
	Tstates += 4;
	SUB(B);
	break;

/* sub c */
case 0x91:
	Tstates += 4;
	SUB(C);
	break;

/* sub d */
case 0x92:
	Tstates += 4;
	SUB(D);
	break;

/* sub e */
case 0x93:
	Tstates += 4;
	SUB(E);
	break;

/* sub h */
case 0x94:
	Tstates += 4;
	SUB(H);
	break;

/* sub l */
case 0x95:
	Tstates += 4;
	SUB(L);
	break;

/* sub (hl) */
case 0x96:
	Tstates += 7;
	work8 = RD_BYTE(HL);
	SUB(work8);
	break;

/* sub a */
case 0x97:
	Tstates += 7;
	A = 0;
	SET(Z_FLAG);
	CLR(S_FLAG);
	CLR(H_FLAG); /* ??? */
	CLR(V_FLAG);
	SET(N_FLAG);
	CLR(C_FLAG);
	break;

/* sbc a,b */
case 0x98:
	Tstates += 4;
	SBC(B);
	break;

/* sbc a,c */
case 0x99:
	Tstates += 4;
	SBC(C);
	break;

/* sbc a,d */
case 0x9A:
	Tstates += 4;
	SBC(D);
	break;

/* sbc a,e */
case 0x9B:
	Tstates += 4;
	SBC(E);
	break;

/* sbc a,h */
case 0x9C:
	Tstates += 4;
	SBC(H);
	break;

/* sbc a,l */
case 0x9D:
	Tstates += 4;
	SBC(L);
	break;

/* sbc a,(hl) */
case 0x9E:
	Tstates += 7;
	work8 = RD_BYTE(HL);
	SBC(work8);
	break;

/* sbc a,a */
case 0x9F:
	Tstates += 4;
	SBC(A);
	break;

/* and b */
case 0xA0:
	Tstates += 4;
	AND(B);
	break;

/* and c */
case 0xA1:
	Tstates += 4;
	AND(C);
	break;

/* and d */
case 0xA2:
	Tstates += 4;
	AND(D);
	break;

/* and e */
case 0xA3:
	Tstates += 4;
	AND(E);
	break;

/* and h */
case 0xA4:
	Tstates += 4;
	AND(H);
	break;

/* and l */
case 0xA5:
	Tstates += 4;
	AND(L);
	break;

/* and (hl) */
case 0xA6:
	Tstates += 7;
	work8 = RD_BYTE(HL);
	AND(work8);
	break;

/* and a */
case 0xA7:
	Tstates += 4;
	AND(A);
	break;

/* xor b */
case 0xA8:
	Tstates += 4;
	XOR(B);
	break;

/* xor c */
case 0xA9:
	Tstates += 4;
	XOR(C);
	break;

/* xor d */
case 0xAA:
	Tstates += 4;
	XOR(D);
	break;

/* xor e */
case 0xAB:
	Tstates += 4;
	XOR(E);
	break;

/* xor h */
case 0xAC:
	Tstates += 4;
	XOR(H);
	break;

/* xor l */
case 0xAD:
	Tstates += 4;
	XOR(L);
	break;

/* xor (hl) */
case 0xAE:
	Tstates += 7;
	work8 = RD_BYTE(HL);
	XOR(work8);
	break;

/* xor a */
case 0xAF:
	Tstates += 4;
	A = 0;
	SET(Z_FLAG);
	CLR(S_FLAG);
	CLR(H_FLAG);
	SET(P_FLAG);
	CLR(N_FLAG);
	CLR(C_FLAG);
	break;

/* or b */
case 0xB0:
	Tstates += 4;
	OR(B);
	break;

/* or c */
case 0xB1:
	Tstates += 4;
	OR(C);
	break;

/* or d */
case 0xB2:
	Tstates += 4;
	OR(D);
	break;

/* or e */
case 0xB3:
	Tstates += 4;
	OR(E);
	break;

/* or h */
case 0xB4:
	Tstates += 4;
	OR(H);
	break;

/* or l */
case 0xB5:
	Tstates += 4;
	OR(L);
	break;

/* or (hl) */
case 0xB6:
	Tstates += 7;
	work8 = RD_BYTE(HL);
	OR(work8);
	break;

/* or a */
case 0xB7:
	Tstates += 4;
	OR(A);
	break;

/* cp b */
case 0xB8:
	Tstates += 4;
	CMP(B);
	break;

/* cp c */
case 0xB9:
	Tstates += 4;
	CMP(C);
	break;

/* cp d */
case 0xBA:
	Tstates += 4;
	CMP(D);
	break;

/* cp e */
case 0xBB:
	Tstates += 4;
	CMP(E);
	break;

/* cp h */
case 0xBC:
	Tstates += 4;
	CMP(H);
	break;

/* cp l */
case 0xBD:
	Tstates += 4;
	CMP(L);
	break;

/* cp (hl) */
case 0xBE:
	Tstates += 7;
	work8 = RD_BYTE(HL);
	CMP(work8);
	break;

/* cp a */
case 0xBF:
	Tstates += 4;
	SET(Z_FLAG);
	CLR(S_FLAG);
	CLR(H_FLAG); /* ??? */
	CLR(V_FLAG);
	SET(N_FLAG);
	CLR(C_FLAG);
	break;

/* ret nz */
case 0xC0:
	Tstates += 5;
	if (!TST(Z_FLAG))
	{
		Tstates += 6;
		POP(PC);
	}
	break;

/* pop bc */
case 0xC1:
	Tstates += 10;
	POP(BC);
	break;
	
/* jp nz,NN */
case 0xC2:
	Tstates += 10;
	if (!TST(Z_FLAG))
	{
		PC = RD_WORD(PC);
	}
	else
	{
		PC += 2;
	}
	break;

/* jp NN */
case 0xC3:
	Tstates += 10;
	PC = RD_WORD(PC);
	break;

/* call nz,NN */
case 0xC4:
	if (!TST(Z_FLAG))
	{
		Tstates += 17;
		CALL;
	}
	else
	{
		Tstates += 10;
		PC += 2;
	}
	break;

/* push bc */
case 0xC5:
	Tstates += 11;
	PUSH(BC);
	break;

/* add a,N */
case 0xC6:
	Tstates += 7;
	work8 = RD_BYTE(PC++);
	ADD(work8);
	break;

/* rst 0x00 */
case 0xC7:
	Tstates += 11;
	RST(0x0000);
	break;

/* ret z */
case 0xC8:
	Tstates += 5;
	if (TST(Z_FLAG))
	{
		Tstates += 6;
		POP(PC);
	}
	break;

/* ret */
case 0xC9:
	Tstates += 10;
	POP(PC);
	break;

/* jp z,NN */
case 0xCA:
	Tstates += 10;
	if (TST(Z_FLAG))
	{
		PC = RD_WORD(PC);
	}
	else
	{
		PC += 2;
	}
	break;

/* operations prefixed by CB */
case 0xCB:
	Z80_M1(op)

#ifndef HAVE_ENOUGH_SWAP
	z80_DecodeCB(op);
#else
	switch (op)
	{
#		include "z80_cbops.c"
	}
#endif /* HAVE_ENOUGH_SWAP */

	break;

/* call z,NN */
case 0xCC:
	if (TST(Z_FLAG))
	{
		Tstates += 17;
		CALL;
	}
	else
	{
		Tstates += 10;
		PC += 2;
	}
	break;

/* call NN */
case 0xCD:
	Tstates += 17;
	CALL;
	break;

/* adc a,N */
case 0xCE:
	Tstates += 7;
	work8 = RD_BYTE(PC++);
	ADC(work8);
	break;

/* rst 0x08 */
case 0xCF:
	Tstates += 11;
	RST(0x0008);
	break;

/* ret nc */
case 0xD0:
	Tstates += 5;
	if (!TST(C_FLAG))
	{
		Tstates += 6;
		POP(PC);
	}
	break;

/* pop de */
case 0xD1:
	Tstates += 10;
	POP(DE);
	break;

/* jp nc,NN */
case 0xD2:
	Tstates += 10;
	if (!TST(C_FLAG))
	{
		PC = RD_WORD(PC);
	}
	else
	{
		PC += 2;
	}
	break;

/* out (N),a */
case 0xD3:
	Tstates += 11;
	OutByte((A << 8) | RD_BYTE(PC++), A);
	break;

/* call nc,NN */
case 0xD4:
	if (!TST(C_FLAG))
	{
		Tstates += 17;
		CALL;
	}
	else
	{
		Tstates += 10;
		PC += 2;
	}
	break;

/* push de */
case 0xD5:
	Tstates += 11;
	PUSH(DE);
	break;

/* sub N */
case 0xD6:
	Tstates += 7;
	work8 = RD_BYTE(PC++);
	SUB(work8);
	break;

/* rst 0x10 */
case 0xD7:
	Tstates += 11;
	RST(0x0010);
	break;

/* ret c */
case 0xD8:
	Tstates += 5;
	if (TST(C_FLAG))
	{
		Tstates += 6;
		POP(PC);
	}
	break;

/* exx */
case 0xD9:
	Tstates += 4;
	work16 = BC; BC = BC1; BC1 = work16;
	work16 = DE; DE = DE1; DE1 = work16;
	work16 = HL; HL = HL1; HL1 = work16;
	break;

/* jp c,NN */
case 0xDA:
	Tstates += 10;
	if (TST(C_FLAG))
	{
		PC = RD_WORD(PC);
	}
	else
	{
		PC += 2;
	}
	break;

/* in a,N */
case 0xDB:
	Tstates += 11;
	A = InByte((A << 8) | RD_BYTE(PC++));
	break;

/* call c,NN */
case 0xDC:
	if (TST(C_FLAG))
	{
		Tstates += 17;
		CALL;
	}
	else
	{
		Tstates += 10;
		PC += 2;
	}
	break;

/* opcode 0xdd - ix register handling */
case 0xDD:
	Z80_M1(op)

#ifndef HAVE_ENOUGH_SWAP
	z80_DecodeDD(op);
#else
	switch (op)
	{
#		define XX  IX
#		define LXX LIX
#		define HXX HIX
#		include "z80_xxops.c"
#		undef XX
#		undef LXX
#		undef HXX
	}
#endif /* HAVE_ENOUGH_SWAP */

	break;

/* sbc a,N */
case 0xDE:
	Tstates += 7;
	work8 = RD_BYTE(PC++);
	SBC(work8);
	break;

/* rst 0x18 */
case 0xDF:
	Tstates += 11;
	RST(0x0018);
	break;

/* ret po */
case 0xE0:
	Tstates += 5;
	if (!TST(P_FLAG))
	{
		Tstates += 6;
		POP(PC);
	}
	break;

/* pop hl */
case 0xE1:
	Tstates += 10;
	POP(HL);
	break;

/* jp po,NN */
case 0xE2:
	Tstates += 10;
	if (!TST(P_FLAG))
	{
		PC = RD_WORD(PC);
	}
	else
	{
		PC += 2;
	}
	break;

/* ex (sp),hl */
case 0xE3:
	Tstates += 19;
	work16 = RD_WORD(SP);
	WR_WORD(SP, HL);
	HL = work16;
	break;

/* call po,NN */
case 0xE4:
	if (!TST(P_FLAG))
	{
		Tstates += 17;
		CALL;
	}
	else
	{
		Tstates += 10;
		PC += 2;
	}
	break;

/* push hl */
case 0xE5:
	Tstates += 11;
	PUSH(HL);
	break;

/* and N */
case 0xE6:
	Tstates += 7;
	work8 = RD_BYTE(PC++);
	AND(work8);
	break;

/* rst 0x20 */
case 0xE7:
	Tstates += 11;
	RST(0x0020);
	break;

/* ret pe */
case 0xE8:
	Tstates += 5;
	if (TST(P_FLAG))
	{
		Tstates += 6;
		POP(PC);
	}
	break;

/* jp (hl) */
case 0xE9:
	Tstates += 4;
	PC = HL;
	break;

/* jp pe,NN */
case 0xEA:
	Tstates += 10;
	if (TST(P_FLAG))
	{
		PC = RD_WORD(PC);
	}
	else
	{
		PC += 2;
	}
	break;

/* ex de,hl */
case 0xEB:
	Tstates += 4;
	work16 = DE, DE = HL, HL = work16;
	break;

/* call pe,NN */
case 0xEC:
	if (TST(P_FLAG))
	{
		Tstates += 17;
		CALL;
	}
	else
	{
		Tstates += 10;
		PC += 2;
	}
	break;

/* operations prefixed by 0xED */
case 0xED:
	Z80_M1(op)

#ifndef HAVE_ENOUGH_SWAP
	z80_DecodeED(op);
#else
	switch (op)
	{
#		include "z80_edops.c"
	}
#endif /* HAVE_ENOUGH_SWAP */

	break;

/* xor N */
case 0xEE:
	Tstates += 7;
	work8 = RD_BYTE(PC++);
	XOR(work8);
	break;

/* rst 0x28 */
case 0xEF:
	Tstates += 11;
	RST(0x0028);
	break;

/* ret p */
case 0xF0:
	Tstates += 5;
	if (!TST(S_FLAG))
	{
		Tstates += 6;
		POP(PC);
	}
	break;

/* pop af */
case 0xF1:
	Tstates += 10;
	POP(AF);
	Z80_RetrieveFlags();
	break;

/* jp p,NN */
case 0xF2:
	Tstates += 10;
	if (!TST(S_FLAG))
	{
		PC = RD_WORD(PC);
	}
	else
	{
		PC += 2;
	}
	break;

/* di */
case 0xF3:
	Tstates += 4;
	IFF1 = 0;
	IFF2 = 0;
	break;

/* call p,NN */
case 0xF4:
	if (!TST(S_FLAG))
	{
		Tstates += 17;
		CALL;
	}
	else
	{
		Tstates += 10;
		PC += 2;
	}
	break;

/* push af */
case 0xF5:
	Tstates += 11;
	Z80_StoreFlags();
	PUSH(AF);
	break;

/* or N */
case 0xF6:
	Tstates += 7;
	work8 = RD_BYTE(PC++);
	OR(work8);
	break;

/* rst 0x30 */
case 0xF7:
	Tstates += 11;
	RST(0x0030);
	break;

/* ret m */
case 0xF8:
	Tstates += 5;
	if (TST(S_FLAG))
	{
		Tstates += 6;
		POP(PC);
	}
	break;

/* ld sp,hl */
case 0xF9:
	Tstates += 6;
	SP = HL;
	break;

/* jp m,NN */
case 0xFA:
	Tstates += 10;
	if (TST(S_FLAG))
	{
		PC = RD_WORD(PC);
	}
	else
	{
		PC += 2;
	}
	break;

/* ei */
case 0xFB:
	/* According to Z80 specs, interrupts are not enabled until AFTER the
	   instruction following the EI. */
	Tstates += 4;
	delayedEI = 1;
	break;

/* call m,NN */
case 0xFC:
	if (TST(S_FLAG))
	{
		Tstates += 17;
		CALL;
	}
	else
	{
		Tstates += 10;
		PC += 2;
	}
	break;

/* opcode 0xfd - iy register handling */
case 0xFD:
	Z80_M1(op)

#ifndef HAVE_ENOUGH_SWAP
	z80_DecodeFD(op);
#else
	switch (op)
	{
#		define NO_RCSID
#		define XX  IY
#		define LXX LIY
#		define HXX HIY
#		include "z80_xxops.c"
#		undef XX
#		undef LXX
#		undef HXX
#		undef NO_RCSID
	}
#endif /* HAVE_ENOUGH_SWAP */

	break;

/* cp N */
case 0xFE:
	Tstates += 7;
	work8  = RD_BYTE(PC++);
	CMP(work8);
	break;

/* rst 0x38 */
case 0xFF:
	Tstates += 11;
	RST(0x0038);
	break;

/* EOF *************************************************************************/
