//	Copyright (c) Jens Kilian 1998
//
//	Permission to use, copy, modify, distribute, and sell this software and its
//	documentation for any purpose is hereby granted without fee, provided that
//	the above copyright notice appear in all copies and that both that
//	copyright notice and this permission notice appear in supporting
//	documentation, and that the name of the copyright holder not be used in
//	advertising or publicity pertaining to distribution of the software without
//	specific, written prior permission.  The copyright holder makes no
//	representations about the suitability of this software for any purpose.  It
//	is provided "as is" without express or implied warranty.
//
//	THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
//	INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
//	EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//	CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
//	DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
//	TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
//	PERFORMANCE OF THIS SOFTWARE.

// File selection dialog management for BeZX.

#ifndef _BZX_ZXFILEDIALOG_H
#define _BZX_ZXFILEDIALOG_H

#include <Entry.h>
#include <FilePanel.h>
#include <Looper.h>
#include <Message.h>
#include <OS.h>

class ZXFileDialog : public BLooper {
	typedef BLooper inherited;
public:
	
	// Constructor, destructor.
	ZXFileDialog(file_panel_mode mode, const char *pLabel);
	virtual ~ZXFileDialog(void);

	// Hook functions.
	virtual void MessageReceived(BMessage *pMessage);
	virtual bool QuitRequested(void);

	// Wait for file selection; return a string which must be free()d.
	char *Go(void);
			
private:
	ZXFileDialog(const ZXFileDialog &orig);
	ZXFileDialog &operator =(const ZXFileDialog &orig);

	// Return an entry_ref for the current directory.
	entry_ref *CurrentDirectory(void) const;
	
	// Static class members.
	static entry_ref smCurrentDirectory;
		
	// Data members.
	BFilePanel mFilePanel;		// the file panel
	BEntry mSelectedEntry;		// the selected entry
	sem_id mFinished;				// synchronizing semaphore
};

#endif // _BZX_ZXFILEDIALOG_H
