//	Copyright (c) Jens Kilian 1998
//
//	Permission to use, copy, modify, distribute, and sell this software and its
//	documentation for any purpose is hereby granted without fee, provided that
//	the above copyright notice appear in all copies and that both that
//	copyright notice and this permission notice appear in supporting
//	documentation, and that the name of the copyright holder not be used in
//	advertising or publicity pertaining to distribution of the software without
//	specific, written prior permission.  The copyright holder makes no
//	representations about the suitability of this software for any purpose.  It
//	is provided "as is" without express or implied warranty.
//
//	THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
//	INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
//	EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//	CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
//	DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
//	TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
//	PERFORMANCE OF THIS SOFTWARE.

//	A view displaying the ZX Spectrum keyboard.

#include <Bitmap.h>
#include <Font.h>
#include <Point.h>
#include <Rect.h>
#include <TranslationUtils.h>
#include <View.h>

#include "ZXConfig.h"
#include "ZXFile.h"
#include "ZXKeyboardView.h"

// Error message shown when image not found.
const char kErrorMessage[] = "Sorry - ZX Spectrum keyboard image not found.";
const float kMessageSize = 24;

ZXKeyboardView::ZXKeyboardView(void)
:	inherited(BRect(B_ORIGIN,B_ORIGIN), "keyboard",
						 B_FOLLOW_ALL, B_WILL_DRAW|B_FULL_UPDATE_ON_RESIZE),
	mpImage(0)
{
	// Try to retrieve the keyboard image.
	ZXFile imageFile(GETCFG(kbdImage), "r");
	mpImage = BTranslationUtils::GetBitmap(&imageFile);
	
	if (mpImage) {
		SetViewColor(B_TRANSPARENT_32_BIT);
	} else {
		SetFont(be_bold_font);
		SetFontSize(kMessageSize);
	}
}

ZXKeyboardView::~ZXKeyboardView(void)
{
	delete mpImage;
	mpImage = 0;
}

void
ZXKeyboardView::Draw(BRect /*updateRect*/)
{
	if (mpImage) {
		// Draw scaled keyboard image.
		DrawBitmap(mpImage, mpImage->Bounds(), Bounds());
	
	} else {
		// Image not available.
		const BRect bounds = Bounds();

		font_height height;
		GetFontHeight(&height);
		const float msgWidth = StringWidth(kErrorMessage);
		const float msgHeight = height.ascent + height.descent;

		const BPoint offset((bounds.left + bounds.right - msgWidth) * 0.5,
								  (bounds.top + bounds.bottom - msgHeight) * 0.5
								  + height.ascent);
		DrawString(kErrorMessage, offset);
	}
}

BPoint
ZXKeyboardView::DefaultSize(void) const
{
	if (mpImage) {
		const BRect bounds = mpImage->Bounds();
		return BPoint(bounds.Width(), bounds.Height());
	}
	
	// No image present, return size of error message.
	font_height height;
	GetFontHeight(&height);
	return BPoint(StringWidth(kErrorMessage),
					  height.ascent + height.descent + height.leading);
}
