//	Copyright (c) Jens Kilian 1998
//
//	Permission to use, copy, modify, distribute, and sell this software and its
//	documentation for any purpose is hereby granted without fee, provided that
//	the above copyright notice appear in all copies and that both that
//	copyright notice and this permission notice appear in supporting
//	documentation, and that the name of the copyright holder not be used in
//	advertising or publicity pertaining to distribution of the software without
//	specific, written prior permission.  The copyright holder makes no
//	representations about the suitability of this software for any purpose.  It
//	is provided "as is" without express or implied warranty.
//
//	THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
//	INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
//	EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//	CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
//	DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
//	TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
//	PERFORMANCE OF THIS SOFTWARE.

//	A window displaying the ZX Spectrum keyboard.

#include <Point.h>
#include <Rect.h>
#include <Window.h>

#include "ZXConfig.h"
#include "ZXKeyboardView.h"
#include "ZXKeyboardWindow.h"

static BPoint
windowSize(BPoint size, ZXKeyboardView *pView)
{
	if (size == B_ORIGIN) {
		// Use default size.
		return pView->DefaultSize();
	}
	return size;
}

ZXKeyboardWindow::ZXKeyboardWindow(
	BPoint location, BPoint size, ZXKeyboardView *pView, const char *pName)
:	inherited(BRect(location, location + (size = windowSize(size, pView))),
				 pName,
				 B_FLOATING_WINDOW_LOOK, B_NORMAL_WINDOW_FEEL, B_AVOID_FOCUS)
{
	// Size and attach the view.
	pView->ResizeTo(size.x, size.y);
	AddChild(pView);
}

ZXKeyboardWindow::~ZXKeyboardWindow(void)
{
	// empty
}

void
ZXKeyboardWindow::FrameMoved(BPoint location)
{
	// Remember current position.
	SETCFG(kbdLocation, location);

	inherited::FrameMoved(location);
}

void
ZXKeyboardWindow::FrameResized(float width, float height)
{
	// Remember current size.
	SETCFG(kbdSize, BPoint(width, height));

	inherited::FrameResized(width, height);
}

bool
ZXKeyboardWindow::QuitRequested(void)
{
	// Just hide the window, and remember the fact.
	Hide();
	SETCFG(kbdShown, false);

	return false;
}
