//	Copyright (c) Jens Kilian 1998
//
//	Permission to use, copy, modify, distribute, and sell this software and its
//	documentation for any purpose is hereby granted without fee, provided that
//	the above copyright notice appear in all copies and that both that
//	copyright notice and this permission notice appear in supporting
//	documentation, and that the name of the copyright holder not be used in
//	advertising or publicity pertaining to distribution of the software without
//	specific, written prior permission.  The copyright holder makes no
//	representations about the suitability of this software for any purpose.  It
//	is provided "as is" without express or implied warranty.
//
//	THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
//	INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
//	EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//	CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
//	DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
//	TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
//	PERFORMANCE OF THIS SOFTWARE.

//	The view class for BeZX.

#ifndef _BZX_ZXVIEW_H
#define _BZX_ZXVIEW_H

#include <Bitmap.h>
#include <OS.h>
#include <Rect.h>
#include <Region.h>
#include <View.h>

class ZXView : public BView {
	typedef BView inherited;
public:

	// Codes for asynchronous events sent to the emulator thread.
	enum {
		kEmulatorReset= 'erst',
		kEmulatorNMI  = 'enmi',
		kSaveSnapshot = 'ssnp',
		kLoadSnapshot = 'lsnp',
		kAudioToggle  = 'atgl',
		kDebugToggle  = 'dtgl',
		kInsertDiskA  = 'dska',
		kInsertDiskB  = 'dskb',
		kSelectModel  = 'smod',
		kScreenDump   = 'sdmp',
		kEmulatorQuit = 'quit'
	};
		
	// Constructor, destructor.
	ZXView(BRect frame, const char *pName);
	virtual ~ZXView(void);
	
	// Hook functions.
	virtual void AttachedToWindow(void);
	virtual void Draw(BRect updateRect);
	virtual void KeyDown(const char *pBytes, int32 numBytes);
	virtual void KeyUp(const char *pBytes, int32 numBytes);
	virtual void Pulse(void);
	
	// Refresh screen.
	void Refresh(BRect area);

	// Change border color.
	void SetBorderColor(uint8 r, uint8 g, uint8 b);

	// Get pointer to buffer bitmap.	
	BBitmap *Buffer(void)
	{
		return mpBuffer;
	}
	
	// Get port to which asynchronous events will be sent.
	port_id EventPort(void)
	{
		return mEventPort;
	}
	
private:
	ZXView(const ZXView &orig);
	ZXView &operator =(const ZXView &orig);
	
	// Process modifier keys.
	void ProcessModifiers(uint32 modifiers);
		
	// Data members.
	BRect mZXScreenFrame;
	BRegion mBorderRegion;
	BBitmap *mpBuffer;
	bool mUnshiftCursorKeys;
	port_id mEventPort;
};

#endif // _BZX_ZXVIEW_H
